/*
 * Copyright (C) 2000-2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */

package com.Ostermiller.bte;

import java.io.*;
import java.util.*;
import java.net.*;

class BTEFile extends Element {

	private BTEFile(){
		super();
	}
    
    BTEFile(Token t) throws CompileException, IOException{
		super(t);
        TagToken tt;
		TagLexer tl = new TagLexer(new ByteArrayInputStream(t.getContents().getBytes()));
		while ((tt = tl.next_token()) != null) {
			if (tt.id == TagToken.NAME){
				if (tt.contents.equalsIgnoreCase("super")){
					if ((tt = tl.next_token()) != null && tt.id == TagToken.VALUE){
						parent = unescape(tt.contents);
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": bte.doc attribute 'super' has no value");
					}

				} else if (tt.contents.equalsIgnoreCase("ext")){
					if ((tt = tl.next_token()) != null  && tt.id == TagToken.VALUE){
						fileExt = unescape(tt.contents);
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": bte.doc attribute 'ext' has no value");
					}

				} else if (tt.contents.equalsIgnoreCase("output")){
					if ((tt = tl.next_token()) != null  && tt.id == TagToken.VALUE){
						if(unescape(tt.contents).equalsIgnoreCase("true")){
                            output = true;
						} else if(unescape(tt.contents).equalsIgnoreCase("false")){
                            output = false;
						} else {
                        	throw new CompileException ("Line " + t.getLineNumber() + ": bte.doc attribute 'output' must have value 'true' or 'false'.");
						}
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": bte.doc attribute 'output' has no value");
					}

				} else {
					warnings.add("Line " + t.getLineNumber() + ": bte.doc has unknown attribute: '" + tt.contents + "'");
				}
			}
        } 
	}  
    
    /** 
	 * The first element that is part of this Template
	 */
	Template main = null;
    
    Template defaultList = null;
    
    Hashtable defaults = new Hashtable();
	
	
    /**
	 * The template that this template gets
	 * inserted into.  This is intended to
	 * form a URL when put into the context
	 * of the URL for this document.
	 */
    String parent = null;
	
	URL parentURL = null;
    
    String fileExt = null;

    boolean output = true;
    
	/** 
	 * The url where this template is located.
	 * Intended to be a hash value for this template
	 */
    URL url = null;

    void print(Writer w) throws IOException {
		if (main != null){
			main.print(w);
		}
        super.print(w);
	}

	void clearWarnings(){
		super.clearWarnings();
		if (main != null){
            main.clearWarnings();
		} 
        if (defaultList != null){
            defaultList.clearWarnings();
		} 
	}
    
    void printWarnings(Writer w, String name, String compiling) throws IOException {
        super.printWarnings(w, name, compiling);
		if (main != null){
            main.printWarnings(w, name, compiling);
		} 
        if (defaultList != null){
            defaultList.printWarnings(w, name, compiling);
		} 
	}
    
    boolean resolve (Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
        boolean resolved = true;
        if (main != null){
            resolved = resolved && main.resolveAll(defaults, included, onDefault, onNoResolve);
		} 
        if (defaultList != null){
            resolved = resolved && defaultList.resolveAll(defaults, included, onDefault, onNoResolve);
		} 
        return resolved;
	}
    
    void hashThyself(Hashtable ht){
        if(main != null){
            main.hashThyself(ht);
        } 
        if (defaultList != null){
            defaultList.hashThyself(ht);
        }
    } 
}
