/*
 * Copyright (C) 2000-2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */

package com.Ostermiller.bte;

import java.io.*;
import java.util.*;

class Element {

	Element(){
	}

	Element(Token t) throws CompileException {
		token = t;
	}

	/** 
	 * The next Element in a linked list of element
	 */
	Element next = null;
	
	/** 
	 * The token from the file that gave rise to this
	 * Element, if any
	 */
	Token token = null;
    
    /** 
	 * Name associated with this element, if any
	 */
    String name = null;
    
    static final int RESOLVE_OK = 0;
    static final int RESOLVE_WARNING = 1;
    static final int RESOLVE_ERROR = 2;
    
    /** 
     * Warning messages;
     */
    protected ArrayList warnings = new ArrayList();
	
	boolean resolve (Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
	    return true;
    }
	
	boolean resolveAll (Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
		boolean resolved = resolve (defaults, included, onDefault, onNoResolve);
		if (next != null){
			resolved = resolved && (next.resolveAll(defaults, included, onDefault, onNoResolve) && resolved);
		}
        return resolved;
	}
    
    void print(Writer w) throws IOException {
		if (token != null && (token.getID() == sym.TEXT || token.getID() == sym.WHITESPACE)){
			w.write(token.getContents());
		}	
		if (next != null){
			next.print(w);
		}
	}

	void clearWarnings(){
        warnings = new ArrayList();
		if (next != null){
			next.clearWarnings();
		}
	}
	
	void printWarnings(Writer w, String name, String compiling) throws IOException {
        if (warnings != null){                   
            Iterator i = warnings.iterator();
            while(i.hasNext()){
                String warning = (String)i.next();
                w.write("Warning in " + name);
			    w.write(System.getProperty("line.separator"));
                w.write(warning);
                w.write(System.getProperty("line.separator"));
                if (compiling != null){
                    w.write("while compiling " + compiling);
                    w.write(System.getProperty("line.separator"));
                }
                w.write(System.getProperty("line.separator"));
            }
        }
		if (next != null){
			next.printWarnings(w, name, compiling);
		}
	}
    
    static String unescape(String s){
		char c = s.charAt(0);
		if (c!='"' && c!='\''){
			return s;
		} 
		StringBuffer sb = new StringBuffer(s.length());
		for (int i=1; i<s.length()-1; i++){
			if (s.charAt(i)=='\\' && s.charAt(i+1)==c){
				i++;
			}
			sb.append(s.charAt(i));
		} 
		return sb.toString();
	}
}
