/*
 * Copyright (C) 2000-2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */

package com.Ostermiller.bte;

import java.io.*;
import java.util.*;

class Group extends Element {

	private Group(){
		super();
	}
    
    int onDefault = Element.RESOLVE_WARNING;
    int onNoResolve = Element.RESOLVE_ERROR;
    boolean resolved = false;
    
    /** 
	 * The first element that is part of this Group
	 */
	Element elements = null;

	Group(Token t) throws CompileException, IOException{
		super(t);
        TagToken tt;
		TagLexer tl = new TagLexer(new ByteArrayInputStream(t.getContents().getBytes()));
		while ((tt = tl.next_token()) != null) {
			if (tt.id == TagToken.NAME){
				if (tt.contents.equalsIgnoreCase("onDefault")){
					if ((tt = tl.next_token()) != null && tt.id == TagToken.VALUE){
						if (unescape(tt.contents).equalsIgnoreCase("ok")){
                            onDefault = Element.RESOLVE_OK;
                        } else if (unescape(tt.contents).equalsIgnoreCase("warning")){
                            onDefault = Element.RESOLVE_WARNING;
                        } else if (unescape(tt.contents).equalsIgnoreCase("error")){
                            onDefault = Element.RESOLVE_ERROR;
                        } else {
                            throw new CompileException ("Line " + t.getLineNumber() + ": group attribute 'onDefault' has invalid value");
                        }
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": group attribute 'onDefault' has no value");
					}					
				} else if (tt.contents.equalsIgnoreCase("onNoResolve")){
					if ((tt = tl.next_token()) != null && tt.id == TagToken.VALUE){
						if (unescape(tt.contents).equalsIgnoreCase("ok")){
                            onNoResolve = Element.RESOLVE_OK;
                        } else if (unescape(tt.contents).equalsIgnoreCase("warning")){
                            onNoResolve = Element.RESOLVE_WARNING;
                        } else if (unescape(tt.contents).equalsIgnoreCase("error")){
                            onNoResolve = Element.RESOLVE_ERROR;
                        } else {
                            throw new CompileException ("Line " + t.getLineNumber() + ": group attribute 'onNoResolve' has invalid value");
                        }
					} else {
						throw new CompileException("Line " + t.getLineNumber() + ": group attribute 'onNoResolve' has no value");
					}					
				} else {
                    throw new CompileException("Line " + t.getLineNumber() + ": group has unknown attribute: '" + tt.contents + "'");
				}
			}
        }       
	} 
    
    boolean resolveWithOutput(Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
        if (elements != null){
            resolved = elements.resolveAll(defaults, included, this.onDefault, this.onNoResolve);
		}
        return resolved;
	}
    
    boolean resolve(Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
        resolveWithOutput(defaults, included, onDefault, onNoResolve);
        return true;
	}
    
    void print(Writer w) throws IOException {
		if (resolved && elements != null){
			elements.print(w);
		}
        super.print(w);
	}
    
    void printWarnings(Writer w, String name, String compiling) throws IOException {  
        super.printWarnings(w, name, compiling);
		if (elements != null){
			elements.printWarnings(w, name, compiling);
		}
    }
	
	void clearWarnings(){
		super.clearWarnings();
		if (elements != null){
			elements.clearWarnings();
		}
	}
}
