/*
 * Copyright (C) 2000-2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */

package com.Ostermiller.bte;

import java.io.*;
import java.util.*;

class IncludeElement extends Element {

	private IncludeElement(){
		super();
	}
    
    private Template template = null;

	IncludeElement(Token t) throws CompileException, IOException{
		super(t);
		TagToken tt;
		TagLexer tl = new TagLexer(new ByteArrayInputStream(t.getContents().getBytes()));
		while ((tt = tl.next_token()) != null) {
			if (tt.id == TagToken.NAME){
				if (tt.contents.equalsIgnoreCase("name")){
					if ((tt = tl.next_token()) != null && tt.id == TagToken.VALUE){
						name = unescape(tt.contents);
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": include attribute 'name' has no value");
					}
					
				} else {
					throw new CompileException("Line " + t.getLineNumber() + ": include has unknown attribute: '" + tt.contents + "'\n");
				}
			}
        } 
		if (name == null){
			throw new CompileException ("Line " + t.getLineNumber() + ": include has no name attribute.");
		}		
	} 
    
    boolean resolve (Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
        template = null;
        if (included.containsKey(name)){
            template = (Template)(included.get(name));
        } else if(defaults.containsKey(name)){
            template = (Template)(defaults.get(name));
            String s = "Line " + token.getLineNumber() + ": include '" + name + "' could not be resolved.";
            switch (onDefault){
                case Element.RESOLVE_OK:
                break;
                case Element.RESOLVE_ERROR:
                    throw new ResolveException(s);
                default:
                    warnings.add(s + " Using default.");
                break;
            }
        } else {
			String s = "Line " + token.getLineNumber() + ": include '" + name + "' could not be resolved.";
            switch (onNoResolve){
                case Element.RESOLVE_OK:
                break;
                case Element.RESOLVE_WARNING:
                    warnings.add(s);
                break;
                default:
                    throw new ResolveException(s);
            }  
            return (false);          
        }
        return (true);
	}
    
    void print(Writer w) throws IOException {
		if (template != null){
		    template.print(w);
		}
        super.print(w);
    }
}
