/*
 * Lexer.java is a generated file.  You probably want to
 * edit Lexer to make changes.  
 */

/*
 * Copyright (C) 2000-2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */
package com.Ostermiller.bte;

import java.io.*;
import java_cup.runtime.*;

%%

%class Lexer
%implements java_cup.runtime.Scanner
%function next_token
%yylexthrow{ 
LexException
%yylexthrow} 
%type java_cup.runtime.Symbol

%{	
    /** 
     * next Token method that allows you to control if whitespace and comments are
     * returned as tokens.
     */
    Symbol next_token(boolean returnComments, boolean returnWhiteSpace)throws IOException, LexException{
        Symbol s = next_token();
        while (s != null && ((!returnWhiteSpace && ((Token)(s.value)).isWhiteSpace()) || (!returnComments && ((Token)(s.value)).isComment()))){
            s = next_token();
        }
        return (s); 
    }

    /**
     * Prints out tokens from a file or System.in.
     * If no arguments are given, System.in will be used for input.
     * If more arguments are given, the first argument will be used as
     * the name of the file to use as input
     *
     * @param args program arguments, of which the first is a filename
     */
    public static void main(String[] args) {
        InputStream in;
        try {
            if (args.length > 0){
                File f = new File(args[0]);
                if (f.exists()){
                    if (f.canRead()){
                        in = new FileInputStream(f);
                    } else {
                        throw new IOException("Could not open " + args[0]);
                    }
                } else {
                    throw new IOException("Could not find " + args[0]);
                }                   
            } else {
                in = System.in;
            }       
            Lexer shredder = new Lexer(in);
            Symbol s;
            while ((s = shredder.next_token()).sym != sym.EOF) {
				Token t = (Token)(s.value);
                System.out.println(t);
            }
        } catch (Exception e){
            System.out.println(e.getMessage());
        }
    }   
%}

%eofval{
  return new Symbol(sym.EOF);
%eofval}

%line
%char
%unicode

Digit=([0-9])
Letter=([a-zA-Z])
BLANK=([ ])
TAB=([\t])
FF=([\f])
CR=([\r])
LF=([\n])
EOL=({CR}|{LF}|{CR}{LF})
WhiteSpace=({BLANK}|{TAB}|{FF}|{EOL})

StringLiteral=(([\"][^\"]*[\"])|([\'][^\']*[\']))
NameToken=(({Letter}|{Digit}|[\.\-])+)
Value=({NameToken}|{StringLiteral})
Attribute=({NameToken}(({WhiteSpace}*)"="({WhiteSpace}*){Value})?)


TagStart=("<%"[Bb][Tt][Ee][\.])
EndTagStart=("<%/"[Bb][Tt][Ee][\.])
TagEnd=("%>")
Template=([Tt][Pp][Ll])
Include=([Ii][Nn][Cc])
Document=([Dd][Oo][Cc])
Group=([Gg][Rr][Pp])
One=([Oo][Nn][Ee])

StartTemplateTag=({TagStart}{Template}(({WhiteSpace}*){Attribute})*({WhiteSpace}*){TagEnd})
EndTemplateTag=({EndTagStart}{Template}{WhiteSpace}*{TagEnd})
StartIncludeTag=({TagStart}{Include}(({WhiteSpace}*){Attribute})*({WhiteSpace}*){TagEnd})
EndIncludeTag=({EndTagStart}{Include}{WhiteSpace}*{TagEnd})
StartDocumentTag=({TagStart}{Document}(({WhiteSpace}*){Attribute})*({WhiteSpace}*){TagEnd})
EndDocumentTag=({EndTagStart}{Document}{WhiteSpace}*{TagEnd})
StartOneTag=({TagStart}{One}(({WhiteSpace}*){Attribute})*({WhiteSpace}*){TagEnd})
EndOneTag=({EndTagStart}{One}{WhiteSpace}*{TagEnd})
StartGroupTag=({TagStart}{Group}(({WhiteSpace}*){Attribute})*({WhiteSpace}*){TagEnd})
EndGroupTag=({EndTagStart}{Group}{WhiteSpace}*{TagEnd})
ErrorTag=(({EndTagStart}|{TagStart})([^\r\n\>\%]*))
NotBTE=([Bb]([^Tt]|([Tt]([^Ee]|([Ee]([^\.]))))))
Other=(([^\<]|([\<]([^\%]|([\%]([^\/Bb]|{NotBTE}|([\/]([^Bb]|{NotBTE})))))))*)


%%

<YYINITIAL> {WhiteSpace}* {
	int ID = sym.WHITESPACE;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {StartTemplateTag} {
	int ID = sym.START_TEMPLATE;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {EndTemplateTag} {
	int ID = sym.END_TEMPLATE;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {StartIncludeTag} {
	int ID = sym.START_INCLUDE;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {EndIncludeTag} {
	int ID = sym.error;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	throw new LexException("Line " + t.getLineNumber() + ": unexpected tag: '" + t.getContents() + "'\n");
}
<YYINITIAL> {StartDocumentTag} {
	int ID = sym.START_DOCUMENT;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {EndDocumentTag} {
	int ID = sym.END_DOCUMENT;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {StartGroupTag} {
	int ID = sym.START_GROUP;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {EndGroupTag} {
	int ID = sym.END_GROUP;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {StartOneTag} {
	int ID = sym.START_ONE;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {EndOneTag} {
	int ID = sym.END_ONE;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}
<YYINITIAL> {ErrorTag} {
	int ID = sym.error;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	throw new LexException("Line " + t.getLineNumber() + ": unknown or malformed bte tag: '" + t.getContents() + "'\n");
}
<YYINITIAL> {Other} {
	int ID = sym.TEXT;
	String text = yytext();
	Token t = (new Token(ID, text, yyline+1, yychar, yychar + text.length()));
	return (new Symbol(ID, t));
}


