/*
 * Copyright (C) 2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */

package com.Ostermiller.bte;

import java.io.*;
import java.util.*;

class One extends Element {

	private One(){
		super();
	}
    
    int onNoResolve = Element.RESOLVE_ERROR;
    Group theOne = null;
    
    /** 
	 * The first element that is part of this One
	 */
	Element elements = null;

	One(Token t) throws CompileException, IOException{
		super(t);
        TagToken tt;
		TagLexer tl = new TagLexer(new ByteArrayInputStream(t.getContents().getBytes()));
		while ((tt = tl.next_token()) != null) {
			if (tt.id == TagToken.NAME){
	            if (tt.contents.equalsIgnoreCase("onNoResolve")){
					if ((tt = tl.next_token()) != null && tt.id == TagToken.VALUE){
						if (unescape(tt.contents).equalsIgnoreCase("ok")){
                            onNoResolve = Element.RESOLVE_OK;
                        } else if (unescape(tt.contents).equalsIgnoreCase("warning")){
                            onNoResolve = Element.RESOLVE_WARNING;
                        } else if (unescape(tt.contents).equalsIgnoreCase("error")){
                            onNoResolve = Element.RESOLVE_ERROR;
                        } else {
                            throw new CompileException ("Line " + t.getLineNumber() + ": one attribute 'onNoResolve' has invalid value");
                        }
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": one attribute 'onNoResolve' has no value");
					}					
				} else {
                    throw new CompileException("Line " + t.getLineNumber() + ": one has unknown attribute: '" + tt.contents + "'");
				}
			}
        }       
	} 
    
    boolean resolve (Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
        for(Element nextOne = elements; nextOne != null; nextOne = nextOne.next){
            Group nextGroup = (Group)nextOne;
            // Override "onNoResolve" to "ok" on all contained groups.
            nextGroup.onNoResolve = Element.RESOLVE_OK;
            if (theOne == null && nextGroup.resolveWithOutput(defaults, included, onDefault, onNoResolve)){
                theOne = (Group)nextOne;
            }
		}
        if (theOne == null){
            String s = "Line " + token.getLineNumber() + ": no groups resolved.";
            switch (onNoResolve){
                case Element.RESOLVE_OK:
                break;
                case Element.RESOLVE_WARNING:
                    warnings.add(s);
                break;
                default:
                    throw new ResolveException(s);
            } 
        }
        return theOne != null;
	}
    
    void print(Writer w) throws IOException {
		if (theOne != null){
			theOne.print(w);
		}
        super.print(w);
	}
    
    void printWarnings(Writer w, String name, String compiling) throws IOException {  
        super.printWarnings(w, name, compiling);
		if (elements != null){
			elements.printWarnings(w, name, compiling);
		}
    }
	
	void clearWarnings(){
		super.clearWarnings();
		if (elements != null){
			elements.clearWarnings();
		}
	}
}
