/* TagLexer.java is a generated file.  You probably want to
 * edit TagLexer to make changes.  
 */

/*
 * Copyright (C) 2000-2001 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */
package com.Ostermiller.bte;

import java.io.*;

%%

%class TagLexer
%function next_token
%type TagToken 

%{

    /**
     * Prints out tokens from a file or System.in.
     * If no arguments are given, System.in will be used for input.
     * If more arguments are given, the first argument will be used as
     * the name of the file to use as input
     *
     * @param args program arguments, of which the first is a filename
     */
    public static void main(String[] args) {
        InputStream in;
        try {
            if (args.length > 0){
                File f = new File(args[0]);
                if (f.exists()){
                    if (f.canRead()){
                        in = new FileInputStream(f);
                    } else {
                        throw new IOException("Could not open " + args[0]);
                    }
                } else {
                    throw new IOException("Could not find " + args[0]);
                }                   
            } else {
                in = System.in;
            }       
            TagLexer shredder = new TagLexer(in);
            TagToken t;
            while ((t = shredder.next_token()) != null) {
                System.out.println(t);
            }
        } catch (IOException e){
            System.out.println(e.getMessage());
        }
    }   
%}

%unicode

%state MID_TAG
%state AFTER_NAME
%state AFTER_EQUAL

Digit=([0-9])
Letter=([a-zA-Z])
AnyChar=([^A]|[A])
Equals=[\=]

StringLiteral=(([\"][^\"]*[\"])|([\'][^\']*[\']))
NameToken=(({Letter}|{Digit}|[\.\-])+)
Value=({NameToken}|{StringLiteral})
Name=({Letter}({Letter}|{Digit}|[\.\-])*)


TagStart=("<%"{Name})

%%


<YYINITIAL> {TagStart} {
	yybegin(MID_TAG);
}

<MID_TAG> {Name} {
	TagToken t = (new TagToken(TagToken.NAME, yytext()));
	yybegin(AFTER_NAME);
	return(t);
}
<AFTER_NAME> {Equals} {
	yybegin(AFTER_EQUAL);
}

<AFTER_EQUAL> {Value} {
	TagToken t = (new TagToken(TagToken.VALUE, yytext()));
	yybegin(MID_TAG);
	return(t);
}

<YYINITIAL, MID_TAG, AFTER_NAME, AFTER_EQUAL> {AnyChar} {
}


