/*
 * Copyright (C) 2000-2004 Stephen Ostermiller
 * http://ostermiller.org/contact.pl?regarding=BTE
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * See COPYING.TXT for details.
 */
 
package com.Ostermiller.bte;
import java.io.*;
import java.util.*;

class Template extends Element {

	private Template(){
		super();
	}
    
    /** 
	 * The first element that is part of this Template
	 */
	Element elements = null;

	Template(Token t) throws CompileException, IOException{
		super(t);
        TagToken tt;
		TagLexer tl = new TagLexer(new ByteArrayInputStream(t.getContents().getBytes()));
		while ((tt = tl.next_token()) != null) {
			if (tt.id == TagToken.NAME){
				if (tt.contents.equalsIgnoreCase("name")){
					if ((tt = tl.next_token()) != null && tt.id == TagToken.VALUE){
						name = unescape(tt.contents);
					} else {
						throw new CompileException ("Line " + t.getLineNumber() + ": template attribute 'name' has no value");
					}
					
				} else {
					warnings.add("Line " + t.getLineNumber() + ": template has unknown attribute: '" + tt.contents + "'");
				}
			}
        }       
	}  
    
    void checkNames(boolean error) throws CompileException {
        if (name == null ){
            String s = "Line " + token.getLineNumber() + ": template has no name attribute.";
            if (error){
                throw new CompileException(s);
            } else {	
                warnings.add(s);
            }
		}
        if (next != null && next instanceof Template){
            ((Template)next).checkNames(error);
        }
    }
    
    void hashThyself(Hashtable ht){
        if (name != null){
            ht.put(name, this);
        } 
        if (next != null && next instanceof Template){
            ((Template)next).hashThyself(ht);
        }
    }
    
    void print(Writer w) throws IOException {
		if (elements != null){
			elements.print(w);
		}
	}
    
    void printWarnings(Writer w, String name, String compiling) throws IOException {  
        super.printWarnings(w, name, compiling);
		if (elements != null){
			elements.printWarnings(w, name, compiling);
		}
    }
	
	void clearWarnings(){
		super.clearWarnings();
		if (elements != null){
			elements.clearWarnings();
		}
	}
    
    boolean resolve (Hashtable defaults, Hashtable included, int onDefault, int onNoResolve) throws ResolveException {
        boolean resolved = true;
		if (elements != null){
            if (name != null){
                // remove this template from the default list for so that 
                // nothing in this template resolves to itself, causing infinite recursion
                defaults.remove(name);  
            } 		
            resolved = (elements.resolveAll(defaults, included, onDefault, onNoResolve));            
            if (name != null){
                // put this template back on the hash table now that we are done resolving
                defaults.put(name, this);
            }
		}
        return resolved;
	}
}
